
# q-expansion of newform 546.2.j.e, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 546
weight = 2
poly_data = [16, 76, 345, 164, 233, 36, 110, 14, 15, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0, 0, 0], 1], [[438544, -66732, -301041, 1003571, -156870, 19422, -67528, 8466, -2241, -503], 2044008], [[3620228, 274164, 1236807, 579550, 644490, -79794, -28346, -34782, 9207, -3064], 1022004], [[-3118196, -3391749, -3196425, -478621, -1723536, -625302, -334006, -12699, -10833, -3977], 1022004], [[-2149816, -9757146, -3491505, -6543167, -967302, -3082518, -375260, -413376, 54315, -27409], 2044008], [[-1074620, -3963994, -2349679, -2887781, -440214, -1521222, -200758, -216152, 29941, -14545], 340668], [[165664, 16896855, 5556402, 10649492, 744876, 5037264, 561428, 695481, -96426, 45928], 1022004], [[5377120, 29957226, 17738031, 20509619, 4653846, 10555542, 1662752, 1399728, -178473, 97465], 2044008], [[742892, 14145267, 5109630, 11604142, 367938, 5520312, 484042, 805587, -117222, 51344], 1022004]]

hecke_ring_character_values = [[365, [1, 0, 0, 0, 0, 0, 0, 0, 0, 0]], [157, [-1, 0, 0, 0, 0, -1, 0, 0, 0, 0]], [379, [-1, 0, 0, 0, 0, -1, 0, 0, 0, 0]]]
aps_data = [[-1, 0, 0, 0, 0, 0, 0, 0, 0, 0], [1, 0, 0, 0, 0, 1, 0, 0, 0, 0], [0, -1, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0, 0, 1], [2, 1, 0, 0, 1, 2, -1, 1, 0, 0], [0, -1, 0, -1, 0, 0, 0, -1, 0, 0], [0, 0, -1, -1, 0, 0, 0, 0, -1, 1], [0, -2, -2, 0, -1, -1, 2, 0, -2, 1], [0, 0, -1, -1, -1, 0, -1, 0, -2, 1], [0, 0, 0, -1, 0, 0, -1, -1, 1, 0], [0, 1, 1, -1, 1, 2, -1, -1, 1, -1], [1, 0, 0, -1, 0, 0, 0, 0, -1, 1], [0, 0, 0, 1, 1, 2, -1, 1, 1, -1], [1, -3, 0, 0, -1, 1, 0, -1, -1, -1], [-1, -1, 0, 0, 2, -1, -1, 1, 1, 1], [0, -1, -1, 1, -1, 3, 2, 1, -2, 1], [0, 0, -4, -2, 1, 0, 1, 0, -1, 2], [0, 1, 1, 0, -1, -3, 0, 0, 0, 1], [1, 0, 0, 0, 1, 1, 0, 2, 1, 1], [4, 3, 0, 0, 2, 4, -2, 0, 0, 0], [0, -3, -3, -3, 1, -2, 0, -3, 0, -1], [-2, -1, 0, 0, -2, -2, 2, 0, 0, 0], [0, 0, 0, 1, 1, 0, 1, 0, 0, 1], [-3, 0, 7, 2, -1, 0, -1, 0, 4, -5], [-2, 0, 0, 0, -1, -2, -1, -2, -2, -2], [-4, 1, 0, 0, -4, -4, 2, -2, -2, -2], [3, 3, 0, 0, 1, 3, 0, 1, 1, 1], [1, 0, 2, 0, 0, 0, 0, 0, 3, -3], [0, 0, 0, 2, 0, -5, 2, 2, -2, 0], [-6, 2, 0, 0, -1, -6, 1, 1, 0, 0], [0, -1, -1, 0, -2, 4, 3, 0, -3, 2], [-5, 1, 0, 0, -1, -5, -1, 2, -2, -2], [1, 0, 5, 2, -1, 0, -1, 0, 2, -3], [1, 0, 0, 0, 0, 1, 1, -3, 1, 1], [0, 2, 2, -4, -2, -9, 1, -4, -1, 2], [0, 6, 6, 2, 4, 2, -4, 2, 4, -4], [0, -4, -4, -3, 0, -3, -1, -3, 1, 0], [0, -3, -3, 1, -1, -10, 0, 1, 0, 1], [0, -2, -2, -2, -5, -2, 2, -2, -2, 5], [0, 1, 1, 1, 0, -3, 4, 1, -4, 0], [-6, 1, 0, 0, -3, -6, 1, -1, -2, -2], [-9, 0, -1, 2, 0, 0, 0, 0, 2, -2], [0, -4, -4, 0, 2, 5, 1, 0, -1, -2], [5, -4, 0, 0, -2, 5, 1, 2, -1, -1], [0, 1, 1, -2, -2, -5, 5, -2, -5, 2], [2, 0, -5, -1, 2, 0, 2, 0, -2, 4], [0, -3, -3, 0, -2, 4, -1, 0, 1, 2], [0, -6, -6, -3, 0, 5, 0, -3, 0, 0], [5, 0, 5, -1, 1, 0, 1, 0, -1, 2], [-1, -5, 0, 0, 2, -1, 1, 0, 3, 3], [-5, 4, 0, 0, -1, -5, 2, 0, 1, 1], [-1, 0, 0, -5, 0, 0, 0, 0, -4, 4], [-7, 0, 1, 3, -4, 0, -4, 0, 2, -6], [-6, 1, 0, 0, 0, -6, -3, 0, -3, -3], [17, 0, -3, 0, -1, 0, -1, 0, -4, 3], [0, -4, 0, 0, 2, 0, 0, 4, 2, 2], [-7, 0, 5, 1, 2, 0, 2, 0, 2, 0], [1, 0, -3, -1, 1, 0, 1, 0, -4, 5], [-9, 0, 2, -2, 4, 0, 4, 0, 2, 2], [-4, 0, -2, -3, 2, 0, 2, 0, 5, -3], [1, -3, 0, 0, 0, 1, -3, 0, -3, -3], [5, -9, 0, 0, -4, 5, 3, -3, -1, -1], [11, 0, -6, -5, 1, 0, 1, 0, -1, 2], [0, 0, 0, 5, 5, 13, -4, 5, 4, -5], [-8, 8, 0, 0, 2, -8, -1, 1, 1, 1], [12, 3, 0, 0, -2, 12, 0, 2, -2, -2], [0, 0, 0, 3, 0, -1, 3, 3, -3, 0], [8, 0, 0, -4, -2, 0, -2, 0, -7, 5], [10, 0, 1, 0, 4, 0, 4, 0, 0, 4], [0, 0, 0, 0, 9, 4, -5, 0, 5, -9], [-1, 5, 0, 0, 4, -1, -4, -1, 0, 0], [-8, -4, 0, 0, 0, -8, 3, -2, 3, 3], [10, -3, 0, 0, 0, 10, 0, 1, 0, 0], [0, 10, 10, 6, 4, 17, -7, 6, 7, -4], [0, 10, 10, 0, 3, -3, -7, 0, 7, -3], [0, -6, -6, -1, 3, 17, -2, -1, 2, -3], [0, 2, 2, 2, 1, -5, 1, 2, -1, -1], [0, 2, 2, 1, -4, 14, 0, 1, 0, 4], [-9, 0, -7, -6, -2, 0, -2, 0, -3, 1], [-9, 0, 3, 3, -2, 0, -2, 0, -5, 3], [0, 4, 4, 1, 9, 11, -8, 1, 8, -9], [-16, 0, 0, 4, -3, 0, -3, 0, 3, -6], [0, 0, 0, 0, -3, 0, 0, -1, -3, -3], [-14, 3, 0, 0, 9, -14, -5, -3, 4, 4], [4, 0, 5, 4, -3, 0, -3, 0, 2, -5], [-4, -2, 0, 0, 1, -4, -2, -3, -1, -1], [2, 10, 0, 0, -1, 2, 1, 7, 0, 0], [8, 0, 8, 5, 4, 0, 4, 0, 3, 1], [-23, -1, 0, 0, 3, -23, -1, 0, 2, 2], [-8, 0, -10, 2, 6, 0, 6, 0, -3, 9], [-13, 0, 0, 0, -1, -13, 2, -4, 1, 1], [2, -2, 0, 0, -2, 2, 1, 6, -1, -1], [5, 0, 0, -5, -2, 0, -2, 0, -7, 5], [0, 0, 0, 4, -4, 8, 2, 4, -2, 4], [4, -8, 0, 0, -8, 4, 4, -3, -4, -4], [-1, 8, 0, 0, 7, -1, -7, 0, 0, 0], [-6, 0, 0, -1, -1, 0, -1, 0, -6, 5], [0, -4, -4, 2, 1, -17, 1, 2, -1, -1], [-10, 0, 10, -1, 0, 0, 0, 0, 6, -6], [-3, -5, 0, 0, 0, -3, 3, -4, 3, 3], [8, 0, 7, 5, 1, 0, 1, 0, 3, -2], [5, -9, 0, 0, -7, 5, 6, -4, -1, -1], [-7, 0, -1, -1, -2, 0, -2, 0, -2, 0], [-9, 0, -5, -6, -4, 0, -4, 0, -3, -1], [0, 4, 4, -6, -2, -3, 2, -6, -2, 2], [0, -4, -4, 0, -9, 6, 7, 0, -7, 9], [0, 0, 0, -1, 2, 23, 2, -1, -2, -2], [5, 7, 0, 0, 4, 5, -3, 1, 1, 1], [0, -5, -5, 0, 3, 33, -1, 0, 1, -3], [0, -1, -1, -5, -4, -12, 2, -5, -2, 4], [0, -8, -8, -1, -3, -15, 5, -1, -5, 3], [-6, 1, 0, 0, 1, -6, 1, 0, 2, 2], [10, -6, 0, 0, 0, 10, 4, 0, 4, 4], [0, 13, 13, 9, 4, -13, -3, 9, 3, -4], [-4, -11, 0, 0, -3, -4, 0, -4, -3, -3], [-20, 0, -2, 1, 3, 0, 3, 0, -2, 5], [-3, 8, 0, 0, -3, -3, 0, -3, -3, -3], [-6, -2, 0, 0, -7, -6, 7, 3, 0, 0], [4, 0, -13, -4, 6, 0, 6, 0, -2, 8], [-9, 10, 0, 0, 6, -9, -6, 1, 0, 0], [9, 3, 0, 0, 6, 9, -10, 1, -4, -4], [0, -4, -4, -3, -1, -9, -1, -3, 1, 1], [-12, 3, 0, 0, 3, -12, 1, -7, 4, 4], [-15, 0, 1, 7, -1, 0, -1, 0, 3, -4], [-21, 0, 1, -1, -1, 0, -1, 0, 0, -1], [5, 0, 2, 6, 0, 0, 0, 0, -3, 3], [0, -6, -6, 1, -5, -28, 4, 1, -4, 5], [0, 5, 5, 1, 4, -11, -8, 1, 8, -4], [-2, 0, -1, -6, -2, 0, -2, 0, -2, 0], [14, 1, 0, 0, -4, 14, 2, 4, -2, -2], [13, 7, 0, 0, 9, 13, -9, -1, 0, 0], [0, 5, 5, 6, 1, 8, -2, 6, 2, -1], [0, 0, 0, 8, 2, 0, 2, 0, -1, 3], [0, -2, -2, -3, -5, 5, 2, -3, -2, 5], [0, -7, -7, -1, -3, 13, 6, -1, -6, 3], [0, -2, -2, -1, 2, 23, -3, -1, 3, -2], [-3, 0, 3, 2, 3, 0, 3, 0, 14, -11], [-2, 0, 1, 4, 3, 0, 3, 0, 0, 3], [-7, -9, 0, 0, -7, -7, 8, 0, 1, 1], [15, 5, 0, 0, 4, 15, -3, -7, 1, 1], [3, 0, 12, -2, -3, 0, -3, 0, 7, -10], [24, 0, 1, -7, 1, 0, 1, 0, -6, 7], [-25, 0, 0, -4, 4, 0, 4, 0, 7, -3], [-10, 0, 0, 8, -2, 0, -2, 0, 0, -2], [6, 11, 0, 0, 9, 6, -7, 6, 2, 2], [4, 3, 0, 0, 3, 4, -4, -1, -1, -1], [10, 0, 5, 4, 1, 0, 1, 0, 1, 0], [0, 10, 10, 1, 4, 20, -3, 1, 3, -4], [-31, -6, 0, 0, -8, -31, 7, -6, -1, -1], [5, 10, 0, 0, 9, 5, -5, -4, 4, 4], [0, -15, -15, -1, -10, -7, 8, -1, -8, 10], [0, 11, 11, 3, 0, 11, -2, 3, 2, 0], [-11, 0, 6, 10, 0, 0, 0, 0, 10, -10], [-6, 0, -3, 2, 7, 0, 7, 0, 1, 6], [-10, -4, 0, 0, -2, -10, 0, -5, -2, -2], [13, 0, -9, -3, 3, 0, 3, 0, -9, 12], [0, -7, -7, -3, -1, 27, 1, -3, -1, 1], [0, -15, -15, -2, -2, 7, -1, -2, 1, 2], [2, 0, -1, -6, 2, 0, 2, 0, -1, 3], [0, -20, -20, -11, -4, 1, 4, -11, -4, 4], [-17, 0, 5, 10, -5, 0, -5, 0, 2, -7], [18, -1, 0, 0, 2, 18, -2, 4, 0, 0], [-4, 0, -8, -1, -5, 0, -5, 0, -7, 2], [0, 13, 13, 3, 10, 6, -5, 3, 5, -10], [23, -6, 0, 0, 4, 23, -7, -1, -3, -3], [0, -3, -3, 1, 3, 3, 4, 1, -4, -3], [12, -6, 0, 0, 0, 12, 5, -1, 5, 5], [-15, 0, 7, 0, -2, 0, -2, 0, 2, -4]]
